@@ -90,7 +90,7 @@ static int client_use(int delta)
     return clients;
 }
 
-int clients_count(void)
+int clients_get_count(void)
 {
     return client_use(0);
 }
@@ -521,7 +521,7 @@ static void client_report_facets(struct client *cl, ZOOM_resultset rs)
                                 ZOOM_facet_field_get_term(facets[facet_idx],
                                                           term_idx, &freq);
                             if (term)
-                                add_facet(se, p, term, freq);
+                                add_facet(se, p, term, freq, cl);
                         }
                         break;
                     }
@@ -543,27 +543,6 @@ static void ingest_raw_record(struct client *cl, ZOOM_record rec)
     client_show_raw_dequeue(cl);
 }
 
-void client_check_preferred_watch(struct client *cl)
-{
-    struct session *se = cl->session;
-    yaz_log(YLOG_DEBUG, "client_check_preferred_watch: %s ", client_get_id(cl));
-    if (se)
-    {
-        client_unlock(cl);
-        /* TODO possible threading issue. Session can have been destroyed */
-        if (session_is_preferred_clients_ready(se)) {
-            session_alert_watch(se, SESSION_WATCH_SHOW_PREF);
-        }
-        else
-            yaz_log(YLOG_DEBUG, "client_check_preferred_watch: Still locked on preferred targets.");
-
-        client_lock(cl);
-    }
-    else
-        yaz_log(YLOG_WARN, "client_check_preferred_watch: %s. No session!", client_get_id(cl));
-
-}
-
 struct suggestions* client_suggestions_create(const char* suggestions_string);
 static void client_suggestions_destroy(struct client *cl);
 
@@ -603,6 +582,11 @@ void client_got_records(struct client *cl)
     struct session *se = cl->session;
     if (se)
     {
+        client_unlock(cl);
+        /* TODO possible threading issue. Session can have been destroyed */
+        if (session_is_preferred_clients_ready(se))
+            session_alert_watch(se, SESSION_WATCH_SHOW_PREF);
+        client_lock(cl);
         if (reclist_get_num_records(se->reclist) > 0)
         {
             client_unlock(cl);
@@ -794,6 +778,8 @@ int client_has_facet(struct client *cl, const char *name)
     for (s = sdb->settings[PZ_FACETMAP]; s; s = s->next)
     {
         const char *p = strchr(s->name + 3, ':');
+        if ( !strncmp(p, ":split:", 7) )
+            p += 6; // PAZ-1009
         if (p && !strcmp(name, p + 1))
             return 1;
     }
@@ -839,6 +825,7 @@ int client_fetch_more(struct client *cl)
     const char *str;
     int extend_recs = 0;
     int number = cl->hits - cl->record_offset;
+    struct connection *co = client_get_connection(cl);
 
     str = session_setting_oneval(sdb, PZ_EXTENDRECS);
     if (!str || !*str)
@@ -846,18 +833,21 @@ int client_fetch_more(struct client *cl)
 
     extend_recs = atoi(str);
 
-    yaz_log(YLOG_LOG, "cl=%s show_stat_no=%d got=%d",
+    yaz_log(YLOG_DEBUG, "cl=%s show_stat_no=%d got=%d",
             client_get_id(cl), cl->show_stat_no, cl->record_offset);
     if (cl->show_stat_no < cl->record_offset)
         return 0;
-    yaz_log(YLOG_LOG, "cl=%s Trying to fetch more", client_get_id(cl));
+    yaz_log(YLOG_DEBUG, "cl=%s Trying to fetch more", client_get_id(cl));
 
     if (number > extend_recs)
         number = extend_recs;
-    if (number > 0)
+    if (number <= 0)
+        yaz_log(YLOG_DEBUG, "cl=%s. OK no more in total set", client_get_id(cl));
+    else if (!co)
+        yaz_log(YLOG_DEBUG, "cl=%s. No connection", client_get_id(cl));
+    else
     {
         ZOOM_resultset set = cl->resultset;
-        struct connection *co = client_get_connection(cl);
 
         str = session_setting_oneval(sdb, PZ_REQUESTSYNTAX);
         ZOOM_resultset_option_set(set, "preferredRecordSyntax", str);
@@ -870,10 +860,6 @@ int client_fetch_more(struct client *cl)
         connection_continue(co);
         return 1;
     }
-    else
-    {
-        yaz_log(YLOG_LOG, "cl=%s. OK no more in total set", client_get_id(cl));
-    }
     return 0;
 }
 
@@ -904,6 +890,22 @@ int client_parse_range(struct client *cl, const char *startrecs,
     return 0;
 }
 
+const char *client_get_query(struct client *cl, const char **type, NMEM nmem)
+{
+    if (cl->pquery)
+    {
+        *type = "pqf";
+        return nmem_strdup(nmem, cl->pquery);
+    }
+    if (cl->cqlquery)
+    {
+        *type = "cql";
+        return nmem_strdup(nmem, cl->cqlquery);
+    }
+    *type = 0;
+    return 0;
+}
+
 int client_start_search(struct client *cl)
 {
     struct session_database *sdb = client_get_database(cl);
@@ -921,11 +923,13 @@ int client_start_search(struct client *cl)
     const char *opt_preferred   = session_setting_oneval(sdb, PZ_PREFERRED);
     const char *extra_args      = session_setting_oneval(sdb, PZ_EXTRA_ARGS);
     const char *opt_present_chunk = session_setting_oneval(sdb, PZ_PRESENT_CHUNK);
+    const char *opt_timeout     = session_setting_oneval(sdb, PZ_TIMEOUT);
     ZOOM_query query;
     char maxrecs_str[24], startrecs_str[24], present_chunk_str[24];
     struct timeval tval;
     int present_chunk = 20; // Default chunk size
     int rc_prep_connection;
+    int operation_timeout = se->service->z3950_operation_timeout;
 
     cl->diagnostic = 0;
     cl->record_failures = cl->ingest_failures = cl->filtered = 0;
@@ -933,12 +937,15 @@ int client_start_search(struct client *cl)
     yaz_gettimeofday(&tval);
     tval.tv_sec += 5;
 
+    if (opt_timeout && *opt_timeout)
+        operation_timeout = atoi(opt_timeout);
+
     if (opt_present_chunk && strcmp(opt_present_chunk,"")) {
         present_chunk = atoi(opt_present_chunk);
         yaz_log(YLOG_DEBUG, "Present chunk set to %d", present_chunk);
     }
     rc_prep_connection =
-        client_prep_connection(cl, se->service->z3950_operation_timeout,
+        client_prep_connection(cl, operation_timeout,
                                se->service->z3950_session_timeout,
                                se->service->server->iochan_man,
                                &tval);
@@ -951,9 +958,10 @@ int client_start_search(struct client *cl)
     }
     else if (!rc_prep_connection)
     {
-        session_log(se, YLOG_LOG, "%s: postponing search: No connection",
-                    client_get_id(cl));
-        client_set_state_nb(cl, Client_Working);
+        client_set_diagnostic(cl, 2,
+                              ZOOM_diag_str(2),
+                              "Cannot create connection");
+        client_set_state_nb(cl, Client_Error);
         return -1;
     }
     co = client_get_connection(cl);
@@ -961,8 +969,6 @@ int client_start_search(struct client *cl)
     link = connection_get_link(co);
     assert(link);
 
-    session_log(se, YLOG_LOG, "%s: new search", client_get_id(cl));
-
     client_destroy_xdoc(cl);
     client_init_xdoc(cl);
 
@@ -1164,6 +1170,12 @@ void client_disconnect(struct client *cl)
     client_set_connection(cl, 0);
 }
 
+void client_mark_dead(struct client *cl)
+{
+    if (cl->connection)
+        connection_mark_dead(cl->connection);
+}
+
 void client_stop(struct client *cl)
 {
     client_lock(cl);
@@ -1319,7 +1331,32 @@ const char *client_get_facet_limit_local(struct client *cl,
     return 0;
 }
 
-static int apply_limit(struct session_database *sdb,
+static void ccl_quote_map_term(CCL_bibset ccl_map, WRBUF w,
+                               const char *term)
+{
+    int quote_it = 0;
+    const char *cp;
+    for (cp = term; *cp; cp++)
+        if ((*cp >= '0' && *cp <= '9') || strchr(" +-", *cp))
+            ;
+        else
+            quote_it = 1;
+    if (!quote_it)
+        wrbuf_puts(w, term);
+    else
+    {
+        wrbuf_putc(w, '\"');
+        for (cp = term; *cp; cp++)
+        {
+            if (strchr( "\\\"", *cp))
+                wrbuf_putc(w, '\\');
+            wrbuf_putc(w, *cp);
+        }
+        wrbuf_putc(w, '\"');
+    }
+}
+
+static int apply_limit(struct client *cl,
                        facet_limits_t facet_limits,
                        WRBUF w_pqf, CCL_bibset ccl_map,
                        struct conf_service *service)
@@ -1328,6 +1365,7 @@ static int apply_limit(struct session_database *sdb,
     int i = 0;
     const char *name;
     const char *value;
+    struct session_database *sdb = client_get_database(cl);
 
     NMEM nmem_tmp = nmem_create();
     for (i = 0; (name = facet_limits_get(facet_limits, i, &value)); i++)
@@ -1347,6 +1385,28 @@ static int apply_limit(struct session_database *sdb,
                 nmem_strsplit_escape2(nmem_tmp, "|", value, &values,
                                       &num, 1, '\\', 1);
 
+                for (i = 0; i < num; i++)
+                {
+                    const char *id = session_lookup_id_facet(cl->session,
+                                                             cl, name,
+                                                             values[i]);
+                    if (id)
+                    {
+                        if ( *id )
+                        {
+                            values[i] = nmem_strdup(nmem_tmp, id);
+                            yaz_log(YLOG_DEBUG,
+                                "apply_limit: s='%s' found id '%s'",s->name,id );
+                        }
+                        else
+                        {
+                            yaz_log(YLOG_DEBUG,
+                                "apply_limit: %s: term '%s' not found, failing client",
+                                s->name, values[i] );
+                            ret = -1;
+                        }
+                    }
+                }
                 nmem_strsplit_escape2(nmem_tmp, ",", s->value, &cvalues,
                                       &cnum, 1, '\\', 1);
 
@@ -1379,10 +1439,8 @@ static int apply_limit(struct session_database *sdb,
                             struct ccl_rpn_node *cn;
                             wrbuf_rewind(ccl_w);
                             wrbuf_puts(ccl_w, ccl);
-                            wrbuf_puts(ccl_w, "=\"");
-                            wrbuf_puts(ccl_w, values[i]);
-                            wrbuf_puts(ccl_w, "\"");
-
+                            wrbuf_putc(ccl_w, '=');
+                            ccl_quote_map_term(ccl_map, ccl_w, values[i]);
                             cn = ccl_find_str(ccl_map, wrbuf_cstr(ccl_w),
                                               &cerror, &cpos);
                             if (cn)
@@ -1465,6 +1523,9 @@ int client_parse_query(struct client *cl, const char *query,
     if (!ccl_map)
         return -3;
 
+    xfree(cl->cqlquery);
+    cl->cqlquery = 0;
+
     w_ccl = wrbuf_alloc();
     wrbuf_puts(w_ccl, query);
 
@@ -1475,17 +1536,23 @@ int client_parse_query(struct client *cl, const char *query,
         wrbuf_puts(w_pqf, " ");
     }
 
-    if (apply_limit(sdb, facet_limits, w_pqf, ccl_map, service))
+    if (apply_limit(cl, facet_limits, w_pqf, ccl_map, service))
     {
+        client_set_state(cl, Client_Error);
         ccl_qual_rm(&ccl_map);
+
+        wrbuf_destroy(w_ccl);
+        wrbuf_destroy(w_pqf);
+
+        xfree(cl->pquery);
+        cl->pquery = 0;
+
         return -2;
     }
 
     facet_limits_destroy(cl->facet_limits);
     cl->facet_limits = facet_limits_dup(facet_limits);
 
-    yaz_log(YLOG_LOG, "Client %s: CCL query: %s limit: %s",
-            client_get_id(cl), wrbuf_cstr(w_ccl), wrbuf_cstr(w_pqf));
     cn = ccl_find_str(ccl_map, wrbuf_cstr(w_ccl), &cerror, &cpos);
     ccl_qual_rm(&ccl_map);
     if (!cn)
@@ -1498,6 +1565,10 @@ int client_parse_query(struct client *cl, const char *query,
                     wrbuf_cstr(w_ccl));
         wrbuf_destroy(w_ccl);
         wrbuf_destroy(w_pqf);
+
+        xfree(cl->pquery);
+        cl->pquery = 0;
+
         return -1;
     }
     wrbuf_destroy(w_ccl);
@@ -1528,7 +1599,7 @@ int client_parse_query(struct client *cl, const char *query,
     {
         if (cl->pquery)
             session_log(se, YLOG_LOG, "Client %s: "
-                        "Re-search due query/limit change: %s to %s", 
+                        "Re-search due query/limit change: %s to %s",
                         client_get_id(cl), cl->pquery, wrbuf_cstr(w_pqf));
         xfree(cl->pquery);
         cl->pquery = xstrdup(wrbuf_cstr(w_pqf));
@@ -1539,14 +1610,10 @@ int client_parse_query(struct client *cl, const char *query,
     }
     wrbuf_destroy(w_pqf);
 
-    xfree(cl->cqlquery);
-    cl->cqlquery = 0;
-
     odr_out = odr_createmem(ODR_ENCODE);
     zquery = p_query_rpn(odr_out, cl->pquery);
     if (!zquery)
     {
-
         session_log(se, YLOG_WARN, "Invalid PQF query for Client %s: %s",
                     client_get_id(cl), cl->pquery);
         ret_value = -1;
@@ -1554,9 +1621,6 @@ int client_parse_query(struct client *cl, const char *query,
     }
     else
     {
-        session_log(se, YLOG_LOG, "PQF for Client %s: %s",
-                    client_get_id(cl), cl->pquery);
-
         /* Support for PQF on SRU targets. */
         if (strcmp(query_syntax, "pqf") != 0 && *sru)
         {
@@ -1590,8 +1654,11 @@ int client_parse_query(struct client *cl, const char *query,
     return ret_value;
 }
 
-int client_parse_sort(struct client *cl, struct reclist_sortparms *sp)
+int client_parse_sort(struct client *cl, struct reclist_sortparms *sp,
+                      int *has_sortmap)
 {
+    if (has_sortmap)
+        *has_sortmap = 0;
     if (sp)
     {
         const char *sort_strategy_and_spec =
@@ -1627,6 +1694,8 @@ int client_parse_sort(struct client *cl, struct reclist_sortparms *sp)
                     xfree(cl->sort_criteria);
                     cl->sort_criteria = xstrdup(p);
                 }
+                if (has_sortmap)
+                    (*has_sortmap)++;
             }
             else {
                 yaz_log(YLOG_LOG, "Client %s: "
@@ -1730,12 +1799,12 @@ int client_get_diagnostic(struct client *cl, const char **message,
     return cl->diagnostic;
 }
 
-const char * client_get_suggestions_xml(struct client *cl, WRBUF wrbuf)
+const char *client_get_suggestions_xml(struct client *cl, WRBUF wrbuf)
 {
     /* int idx; */
     struct suggestions *suggestions = cl->suggestions;
 
-    if (!suggestions) 
+    if (!suggestions)
         return "";
     if (suggestions->passthrough)
     {
@@ -1759,7 +1828,6 @@ const char * client_get_suggestions_xml(struct client *cl, WRBUF wrbuf)
     return wrbuf_cstr(wrbuf);
 }
 
-
 void client_set_database(struct client *cl, struct session_database *db)
 {
     cl->database = db;
