@@ -53,10 +53,10 @@ struct conf_config
     struct conf_server *servers;
 
     int no_threads;
+    int max_sockets;
     WRBUF confdir;
     char *path;
     iochan_man_t iochan_man;
-    database_hosts_t database_hosts;
 };
 
 struct service_xslt
@@ -170,7 +170,8 @@ static struct conf_metadata* conf_service_add_metadata(
     enum conf_metadata_mergekey mt,
     const char *facetrule,
     const char *limitmap,
-    const char *limitcluster
+    const char *limitcluster,
+    const char *icurule
     )
 {
     struct conf_metadata * md = 0;
@@ -202,6 +203,7 @@ static struct conf_metadata* conf_service_add_metadata(
     md->facetrule = nmem_strdup_null(nmem, facetrule);
     md->limitmap = nmem_strdup_null(nmem, limitmap);
     md->limitcluster = nmem_strdup_null(nmem, limitcluster);
+    md->icurule = nmem_strdup_null(nmem, icurule);
     return md;
 }
 
@@ -315,6 +317,7 @@ static int parse_metadata(struct conf_service *service, xmlNode *n,
     xmlChar *xml_limitmap = 0;
     xmlChar *xml_limitcluster = 0;
     xmlChar *xml_icu_chain = 0;
+    xmlChar *xml_icurule = 0;
 
     struct _xmlAttr *attr;
 
@@ -358,6 +361,9 @@ static int parse_metadata(struct conf_service *service, xmlNode *n,
         else if (!xmlStrcmp(attr->name, BAD_CAST "limitcluster") &&
                  attr->children && attr->children->type == XML_TEXT_NODE)
             xml_limitcluster = attr->children->content;
+        else if (!xmlStrcmp(attr->name, BAD_CAST "icurule") &&
+                 attr->children && attr->children->type == XML_TEXT_NODE)
+            xml_icurule = attr->children->content;
         else
         {
             yaz_log(YLOG_FATAL, "Unknown metadata attribute '%s'", attr->name);
@@ -515,7 +521,9 @@ static int parse_metadata(struct conf_service *service, xmlNode *n,
                               mergekey_type,
                               (const char *) xml_icu_chain,
                               (const char *) xml_limitmap,
-                              (const char *) xml_limitcluster);
+                              (const char *) xml_limitcluster,
+                              (const char *) xml_icurule
+        );
     (*md_node)++;
     return 0;
 }
@@ -872,7 +880,6 @@ static struct conf_server *server_create(struct conf_config *config,
     server->charsets = 0;
     server->http_server = 0;
     server->iochan_man = 0;
-    server->database_hosts = config->database_hosts;
     server->settings_fname = 0;
 
     if (server_id)
@@ -1075,6 +1082,9 @@ static void info_service_metadata(struct conf_service *service, WRBUF w)
                 case Metadata_type_position:
                     wrbuf_puts(w, "position");
                     break;
+                case Metadata_type_retrieval:
+                    wrbuf_puts(w, "retrieval");
+                    break;
                 default:
                     wrbuf_puts(w, "yes");
                     break;
@@ -1095,8 +1105,7 @@ static void info_service_metadata(struct conf_service *service, WRBUF w)
             case Metadata_type_float:
                 wrbuf_puts(w, " type=\"float\"");
                 break;
-            case Metadata_type_relevance:
-            case Metadata_type_position:
+            default:
                 break;
             }
             switch (md->merge)
@@ -1137,15 +1146,14 @@ static void info_service_metadata(struct conf_service *service, WRBUF w)
 
 static void info_service_databases(struct conf_service *service, WRBUF w)
 {
-    struct database *db;
-    struct setting *s;
-    int i;
 
     if (service->databases)
     {
+        struct database *db;
         wrbuf_puts(w, "   <databases>\n");
-        for(db = service->databases; db; db = db->next)
+        for (db = service->databases; db; db = db->next)
         {
+            int i;
             wrbuf_puts(w, "    <database");
             if (db->id)
             {
@@ -1156,18 +1164,16 @@ static void info_service_databases(struct conf_service *service, WRBUF w)
             wrbuf_puts(w, ">\n");
             for (i = 0; i < db->num_settings; i++)
             {
-                s = db->settings[i];
-                while (s != NULL)
+                struct setting *s = db->settings[i];
+                for (; s; s = s->next)
                 {
-                    wrbuf_puts(w, "     <setting");
-                    wrbuf_puts(w, " name=\"");
+                    wrbuf_puts(w, "     <setting name=\"");
                     wrbuf_xmlputs(w, s->name);
                     wrbuf_puts(w, "\"");
                     wrbuf_puts(w, " value=\"");
                     wrbuf_xmlputs(w, s->value);
                     wrbuf_puts(w, "\"");
                     wrbuf_puts(w, " />\n");
-                    s = s->next;
                 }
             }
             wrbuf_puts(w, "    </database>\n");
@@ -1180,7 +1186,6 @@ void info_services(struct conf_server *server, WRBUF w)
 {
     struct conf_service *s = server->service;
     int i;
-    struct setting *S;
 
     wrbuf_puts(w, " <services>\n");
     for (; s; s = s->next)
@@ -1195,34 +1200,30 @@ void info_services(struct conf_server *server, WRBUF w)
         wrbuf_puts(w, ">\n");
         if (s->settings)
         {
-            for (i=0; i<s->settings->num_settings; i++)
+            for (i = 0; i < s->settings->num_settings; i++)
             {
-                S = s->settings->settings[i];
-                while (S != NULL) {
-                    wrbuf_puts(w, "   <setting");
-                    wrbuf_puts(w, " name=\"");
+                struct setting *S = s->settings->settings[i];
+                for (; S; S = S->next)
+                {
+                    wrbuf_puts(w, "   <setting name=\"");
                     wrbuf_xmlputs(w,  S->name);
                     wrbuf_puts(w, "\"");
                     wrbuf_puts(w, " value=\"");
                     wrbuf_xmlputs(w, S->value);
                     wrbuf_puts(w, "\"");
-                    if (S->target) {
+                    if (S->target)
+                    {
                         wrbuf_puts(w, " target=\"");
                         wrbuf_xmlputs(w, S->target);
                         wrbuf_puts(w, "\"");
                     }
-
                     wrbuf_puts(w, " />\n");
-
-                    S = S->next;
                 }
             }
         }
         info_service_metadata(s, w);
         info_service_databases(s, w);
-        wrbuf_puts(w, "  </service>");
-
-        wrbuf_puts(w, "\n");
+        wrbuf_puts(w, "  </service>\n");
     }
     wrbuf_puts(w, " </services>\n");
 }
@@ -1252,6 +1253,15 @@ static int parse_config(struct conf_config *config, xmlNode *root)
                 xmlFree(number);
             }
         }
+        else if (!strcmp((const char *) n->name, "sockets"))
+        {
+            xmlChar *number = xmlGetProp(n, (xmlChar *) "max");
+            if (number)
+            {
+                config->max_sockets = atoi((const char *) number);
+                xmlFree(number);
+            }
+        }
         else if (!strcmp((const char *) n->name, "file"))
         {
             xmlChar *path = xmlGetProp(n, (xmlChar *) "path");
@@ -1309,8 +1319,8 @@ struct conf_config *config_create(const char *fname)
     config->servers = 0;
     config->path = nmem_strdup(nmem, ".");
     config->no_threads = 0;
+    config->max_sockets = 0;
     config->iochan_man = 0;
-    config->database_hosts = database_hosts_create();
 
     config->confdir = wrbuf_alloc();
     if ((p = strrchr(fname,
@@ -1330,12 +1340,14 @@ struct conf_config *config_create(const char *fname)
     r = yaz_xml_include_simple(n, wrbuf_cstr(config->confdir));
     if (r == 0) /* OK */
     {
+#ifndef WIN32
         yaz_log(YLOG_LOG, "Configuration %s after include processing",
                 fname);
 #if LIBXML_VERSION >= 20600
         xmlDocFormatDump(yaz_log_file(), doc, 0);
 #else
         xmlDocDump(yaz_log_file(), doc);
+#endif
 #endif
         r = parse_config(config, n);
     }
@@ -1373,7 +1385,6 @@ void config_destroy(struct conf_config *config)
             struct conf_server *s_next = server->next;
             server_destroy(server);
             server = s_next;
-            database_hosts_destroy(&config->database_hosts);
         }
         wrbuf_destroy(config->confdir);
         nmem_destroy(config->nmem);
@@ -1411,7 +1422,7 @@ int config_start_listeners(struct conf_config *conf,
 {
     struct conf_server *ser;
 
-    conf->iochan_man = iochan_man_create(conf->no_threads);
+    conf->iochan_man = iochan_man_create(conf->no_threads, conf->max_sockets);
     for (ser = conf->servers; ser; ser = ser->next)
     {
         WRBUF w;
