@@ -28,7 +28,6 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #include "charsets.h"
 #include "http.h"
 #include "database.h"
-#include "host.h"
 
 enum conf_metadata_type {
     Metadata_type_generic,    // Generic text field
@@ -38,6 +37,7 @@ enum conf_metadata_type {
     Metadata_type_skiparticle,
     Metadata_type_relevance,
     Metadata_type_position,
+    Metadata_type_retrieval,
 };
 
 enum conf_metadata_merge {
@@ -82,6 +82,7 @@ struct conf_metadata
 
     char *limitmap;  // Should be expanded into service-wide default e.g. pz:limitmap:<name>=value setting
     char *limitcluster;
+    char *icurule;
 };
 
 
@@ -154,7 +155,6 @@ struct conf_server
     struct conf_config *config;
     http_server_t http_server;
     iochan_man_t iochan_man;
-    database_hosts_t database_hosts;
 };
 
 struct conf_config *config_create(const char *fname);
